﻿Imports System.Runtime.CompilerServices

Namespace System.IO

   Public Module StreamX

      ''' <summary> kopiert von einem Stream in einen anderen </summary>
      ''' <remarks> 
      ''' Es gibt Streams ohne festgelegtes Ende (zB. NetworkStream).
      ''' In solchem Fall **muß** 'count' angegeben werden.
      ''' </remarks>
      <Extension()> _
      Public Sub WriteTo( _
                ByVal readStream As Stream, _
                ByVal writeStream As Stream, _
                Optional ByVal count As Long = -1, _
                Optional ByVal bufSize As Integer = 1024)
         Dim buf(bufSize - 1) As Byte
         If count < 0 AndAlso readStream.CanSeek Then
            count = readStream.Length - readStream.Position
         End If
         If count < 0 Then
            ' Durch 0-Byte-Lesevorgang terminierte Kopier-Schleife
            ' Ein NetworkStream würde ein Timeout-Problem verursachen
            Do
               Dim portion = readStream.Read(buf, 0, bufSize)
               If portion = 0 Then Return
               writeStream.Write(buf, 0, portion)
            Loop
         Else
            ' zähler-gesteuerte Kopier-Schleife
            Do
               If count < bufSize Then bufSize = CInt(count)
               Dim portion = readStream.Read(buf, 0, bufSize)
               If portion = 0 Then Throw New ArgumentException( _
                  "Die angegebene Anzahl Bytes konnte nicht aus dem Lese-Stream gelesen werden", _
                  "readStream + count")
               count -= portion
               writeStream.Write(buf, 0, portion)
            Loop Until count = 0
         End If
      End Sub

   End Module

End Namespace
