﻿Imports System.Data
Imports System.Windows.Forms
Imports System.Data.Common
Imports System.ComponentModel

Public Class BindingSourceInfos

   Public Class EventArg : Inherits EventArgs
      Public ParentRow As DataRow
      Public ChildTables As DataTable()
   End Class
   Public Event TablesNeedsFill As EventHandler(Of EventArg)

   Private _SourceInfos As New Dictionary(Of BindingSource, BindingSourceInfo)
   Public Incrementals As New List(Of BindingSource)
   Private _Dataset As DataSet

   Default Public ReadOnly Property Item(ByVal bs As BindingSource) As BindingSourceInfo
      Get
         Return _SourceInfos(bs)
      End Get
   End Property

   Public Sub Add(ByVal bs As BindingSource)
      Dim tb = bs.DataTable
      If tb.Null Then Return
      If _Dataset.Null Then
         _Dataset = tb.DataSet
      ElseIf tb.DataSet IsNot _Dataset Then
         Throw Me.Exception("die zuzufügenden bindingSource is an ein anderes Dataset gebunden")
      End If
      Dim inf = New BindingSourceInfo With {.BindingSource = bs, .Table = tb}
      Dim ConnectedOnce As Boolean = False
      For Each inf2 In _SourceInfos.Values
         Dim bs2 = inf2.BindingSource
         If bs2.DataSource Is bs Then
            Try
               If ConnectedOnce Then Exit For
               ConnectedOnce = True
            Finally
               inf.Add(inf2)
            End Try
         ElseIf bs.DataSource Is bs2 Then
            Try
               If ConnectedOnce Then Exit For
               ConnectedOnce = True
            Finally
               inf2.Add(inf)
            End Try
         End If
      Next
      AddHandler bs.CurrentChanged, AddressOf BindingSource_CurrentChanged
      AddHandler bs.Disposed, AddressOf BindingSource_Disposed
      _SourceInfos.Add(bs, inf)
   End Sub

   Private Sub BindingSource_Disposed(ByVal sender As Object, ByVal e As System.EventArgs)
      Dim bs = DirectCast(sender, BindingSource)
      RemoveHandler bs.CurrentChanged, AddressOf BindingSource_CurrentChanged
      RemoveHandler bs.Disposed, AddressOf BindingSource_Disposed
      _SourceInfos.Remove(bs)
   End Sub

   Private Sub BindingSource_CurrentChanged(ByVal sender As Object, ByVal e As System.EventArgs)
      Dim bs = DirectCast(sender, BindingSource)
      If bs.Position < 0 OrElse Not CheckIncremental(bs) Then Return
      'die BindingSources aller ChildTables unpluggen, es sei denn, sie sind nicht related, oder sie sind ChildSources von dieser hier.
      Dim childs = _SourceInfos(bs)
      For Each inf In childs
         Dim bsChilds = TableEventEnabler.Enablers(inf.Table).BindingSources
         For Each bsChild In bsChilds
            If bsChild Is inf.BindingSource Then
            ElseIf _Dataset.Relations.Contains(bsChild.DataMember) Then
               _SourceInfos(bsChild).Plug = False
            End If
         Next
         inf.Plug = True
      Next
      'Befüllung der Tables aller ChildSources anfordern.
      Dim rwParent = bs.At
      Dim toFill = childs.Select(Function(c) c.Table).ToArray
      RaiseEvent TablesNeedsFill(Me, New EventArg With {.ChildTables = toFill, .ParentRow = rwParent})
   End Sub

   'checken, ob bs oder Parent incrementell befüllt.
   Private Function CheckIncremental(ByVal bss As BindingSource) As Boolean
      Do Until bss Is Nothing
         If Incrementals.Contains(bss) Then Return True
         bss = TryCast(bss.DataSource, BindingSource)
      Loop
      Return False
   End Function

End Class
